@@ -101,13 +101,13 @@ struct _ThunarAbstractIconViewPrivate
   gint ui_merge_id;
 
   /* mouse gesture support */
-  gint gesture_start_x;
-  gint gesture_start_y;
-  gint gesture_current_x;
-  gint gesture_current_y;
-  gint gesture_expose_id;
-  gint gesture_motion_id;
-  gint gesture_release_id;
+  gint   gesture_start_x;
+  gint   gesture_start_y;
+  gint   gesture_current_x;
+  gint   gesture_current_y;
+  gulong gesture_expose_id;
+  gulong gesture_motion_id;
+  gulong gesture_release_id;
 };
 
 
@@ -418,12 +418,12 @@ thunar_abstract_icon_view_gesture_action (ThunarAbstractIconView *abstract_icon_
   else if (abstract_icon_view->priv->gesture_start_x - abstract_icon_view->priv->gesture_current_x > 40
       && ABS (abstract_icon_view->priv->gesture_start_y - abstract_icon_view->priv->gesture_current_y) < 40)
     {
-      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/back");
+      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/placeholder-go-history-actions/back");
     }
   else if (abstract_icon_view->priv->gesture_current_x - abstract_icon_view->priv->gesture_start_x > 40
       && ABS (abstract_icon_view->priv->gesture_start_y - abstract_icon_view->priv->gesture_current_y) < 40)
     {
-      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/forward");
+      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/placeholder-go-history-actions/forward");
     }
   else if (abstract_icon_view->priv->gesture_current_y - abstract_icon_view->priv->gesture_start_y > 40
       && ABS (abstract_icon_view->priv->gesture_start_x - abstract_icon_view->priv->gesture_current_x) < 40)
