@@ -681,14 +681,10 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
       selection = gtk_tree_view_get_selection (tree_view);
       if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL))
         {
-          /* select the path on which the user clicked if not selected yet */
+          /* show the current directory menu if a user right-clicks an
+           * uselected item in the view. */
           if (!gtk_tree_selection_path_is_selected (selection, path))
-            {
-              /* we don't unselect all other items if Control is active */
-              if ((event->state & GDK_CONTROL_MASK) == 0)
-                gtk_tree_selection_unselect_all (selection);
-              gtk_tree_selection_select_path (selection, path);
-            }
+            gtk_tree_selection_unselect_all (selection);
           gtk_tree_path_free (path);
         }
 
