@@ -49,6 +49,7 @@ enum
   PROP_FOLDERS_FIRST,
   PROP_NUM_FILES,
   PROP_SHOW_HIDDEN,
+  N_PROPERTIES
 };
 
 /* Signal identifiers */
@@ -69,6 +70,7 @@ typedef gint (*ThunarSortFunc) (const ThunarFile *a,
 static void               thunar_list_model_tree_model_init       (GtkTreeModelIface      *iface);
 static void               thunar_list_model_drag_dest_init        (GtkTreeDragDestIface   *iface);
 static void               thunar_list_model_sortable_init         (GtkTreeSortableIface   *iface);
+static void               thunar_list_model_dispose               (GObject                *object);
 static void               thunar_list_model_finalize              (GObject                *object);
 static void               thunar_list_model_get_property          (GObject                *object,
                                                                    guint                   prop_id,
@@ -229,7 +231,8 @@ struct _ThunarListModel
 
 
 
-static guint list_model_signals[LAST_SIGNAL];
+static guint       list_model_signals[LAST_SIGNAL];
+static GParamSpec *list_model_props[N_PROPERTIES] = { NULL, };
 
 
 
@@ -246,6 +249,7 @@ thunar_list_model_class_init (ThunarListModelClass *klass)
   GObjectClass *gobject_class;
 
   gobject_class               = G_OBJECT_CLASS (klass);
+  gobject_class->dispose      = thunar_list_model_dispose;
   gobject_class->finalize     = thunar_list_model_finalize;
   gobject_class->get_property = thunar_list_model_get_property;
   gobject_class->set_property = thunar_list_model_set_property;
@@ -255,79 +259,76 @@ thunar_list_model_class_init (ThunarListModelClass *klass)
    *
    * Tells whether the sorting should be case sensitive.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_CASE_SENSITIVE,
-                                   g_param_spec_boolean ("case-sensitive",
-                                                         "case-sensitive",
-                                                         "case-sensitive",
-                                                         TRUE,
-                                                         EXO_PARAM_READWRITE));
+  list_model_props[PROP_CASE_SENSITIVE] =
+      g_param_spec_boolean ("case-sensitive",
+                            "case-sensitive",
+                            "case-sensitive",
+                            TRUE,
+                            EXO_PARAM_READWRITE);
 
   /**
    * ThunarListModel:date-style:
    *
    * The style used to format dates.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_DATE_STYLE,
-                                   g_param_spec_enum ("date-style",
-                                                      "date-style",
-                                                      "date-style",
-                                                      THUNAR_TYPE_DATE_STYLE,
-                                                      THUNAR_DATE_STYLE_SIMPLE,
-                                                      EXO_PARAM_READWRITE));
+  list_model_props[PROP_DATE_STYLE] =
+      g_param_spec_enum ("date-style",
+                         "date-style",
+                         "date-style",
+                         THUNAR_TYPE_DATE_STYLE,
+                         THUNAR_DATE_STYLE_SIMPLE,
+                         EXO_PARAM_READWRITE);
 
   /**
    * ThunarListModel:folder:
    *
    * The folder presented by this #ThunarListModel.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_FOLDER,
-                                   g_param_spec_object ("folder",
-                                                        "folder",
-                                                        "folder",
-                                                        THUNAR_TYPE_FOLDER,
-                                                        EXO_PARAM_READWRITE));
+  list_model_props[PROP_FOLDER] =
+      g_param_spec_object ("folder",
+                           "folder",
+                           "folder",
+                           THUNAR_TYPE_FOLDER,
+                           EXO_PARAM_READWRITE);
 
   /**
    * ThunarListModel::folders-first:
    *
    * Tells whether to always sort folders before other files.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_FOLDERS_FIRST,
-                                   g_param_spec_boolean ("folders-first",
-                                                         "folders-first",
-                                                         "folders-first",
-                                                         TRUE,
-                                                         EXO_PARAM_READWRITE));
+  list_model_props[PROP_FOLDERS_FIRST] =
+      g_param_spec_boolean ("folders-first",
+                            "folders-first",
+                            "folders-first",
+                            TRUE,
+                            EXO_PARAM_READWRITE);
 
   /**
    * ThunarListModel::num-files:
    *
    * The number of files in the folder presented by this #ThunarListModel.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_NUM_FILES,
-                                   g_param_spec_uint ("num-files",
-                                                      "num-files",
-                                                      "num-files",
-                                                      0, G_MAXUINT, 0,
-                                                      EXO_PARAM_READABLE));
+  list_model_props[PROP_NUM_FILES] =
+      g_param_spec_uint ("num-files",
+                         "num-files",
+                         "num-files",
+                         0, G_MAXUINT, 0,
+                         EXO_PARAM_READABLE);
 
   /**
    * ThunarListModel::show-hidden:
    *
    * Tells whether to include hidden (and backup) files.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_SHOW_HIDDEN,
-                                   g_param_spec_boolean ("show-hidden",
-                                                         "show-hidden",
-                                                         "show-hidden",
-                                                         FALSE,
-                                                         EXO_PARAM_READWRITE));
+  list_model_props[PROP_SHOW_HIDDEN] =
+      g_param_spec_boolean ("show-hidden",
+                            "show-hidden",
+                            "show-hidden",
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
+  /* install properties */
+  g_object_class_install_properties (gobject_class, N_PROPERTIES, list_model_props);
 
   /**
    * ThunarListModel::error:
@@ -416,14 +417,22 @@ thunar_list_model_init (ThunarListModel *store)
 
 
 
+static void
+thunar_list_model_dispose (GObject *object)
+{
+  /* unlink from the folder (if any) */
+  thunar_list_model_set_folder (THUNAR_LIST_MODEL (object), NULL);
+
+  (*G_OBJECT_CLASS (thunar_list_model_parent_class)->dispose) (object);
+}
+
+
+
 static void
 thunar_list_model_finalize (GObject *object)
 {
   ThunarListModel *store = THUNAR_LIST_MODEL (object);
 
-  /* unlink from the folder (if any) */
-  thunar_list_model_set_folder (store, NULL);
-
   g_sequence_free (store->rows);
 
   /* disconnect from the file monitor */
@@ -1197,6 +1206,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
   gint          *indices;
   GSequenceIter *row;
   GList         *lp;
+  gboolean       has_handler;
 
   /* we use a simple trick here to avoid allocating
    * GtkTreePath's again and again, by simply accessing
@@ -1206,6 +1216,9 @@ thunar_list_model_files_added (ThunarFolder    *folder,
   path = gtk_tree_path_new_first ();
   indices = gtk_tree_path_get_indices (path);
 
+  /* check if we have any handlers connected for "row-inserted" */
+  has_handler = g_signal_has_handler_pending (G_OBJECT (store), store->row_inserted_id, 0, FALSE);
+
   /* process all added files */
   for (lp = files; lp != NULL; lp = lp->next)
     {
@@ -1224,11 +1237,14 @@ thunar_list_model_files_added (ThunarFolder    *folder,
           row = g_sequence_insert_sorted (store->rows, file,
                                           thunar_list_model_cmp_func, store);
 
-          /* generate an iterator for the new item */
-          GTK_TREE_ITER_INIT (iter, store->stamp, row);
+          if (has_handler)
+            {
+              /* generate an iterator for the new item */
+              GTK_TREE_ITER_INIT (iter, store->stamp, row);
 
-          indices[0] = g_sequence_iter_get_position (row);
-          gtk_tree_model_row_inserted (GTK_TREE_MODEL (store), path, &iter);
+              indices[0] = g_sequence_iter_get_position (row);
+              gtk_tree_model_row_inserted (GTK_TREE_MODEL (store), path, &iter);
+            }
         }
     }
 
@@ -1236,7 +1252,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
   gtk_tree_path_free (path);
 
   /* number of visible files may have changed */
-  g_object_notify (G_OBJECT (store), "num-files");
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_NUM_FILES]);
 }
 
 
@@ -1297,7 +1313,7 @@ thunar_list_model_files_removed (ThunarFolder    *folder,
     }
 
   /* this probably changed */
-  g_object_notify (G_OBJECT (store), "num-files");
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_NUM_FILES]);
 }
 
 
@@ -1406,8 +1422,8 @@ sort_by_mime_type (const ThunarFile *a,
   const gchar *content_type_b;
   gint         result;
 
-  content_type_a = thunar_file_get_content_type (a);
-  content_type_b = thunar_file_get_content_type (b);
+  content_type_a = thunar_file_get_content_type (THUNAR_FILE (a));
+  content_type_b = thunar_file_get_content_type (THUNAR_FILE (b));
 
   if (content_type_a == NULL)
     content_type_a = "";
@@ -1535,7 +1551,7 @@ sort_by_type (const ThunarFile *a,
     }
   else
     {
-      content_type_a = thunar_file_get_content_type (a);
+      content_type_a = thunar_file_get_content_type (THUNAR_FILE (a));
       description_a = g_content_type_get_description (content_type_a);
     }
 
@@ -1546,7 +1562,7 @@ sort_by_type (const ThunarFile *a,
     }
   else
     {
-      content_type_b = thunar_file_get_content_type (b);
+      content_type_b = thunar_file_get_content_type (THUNAR_FILE (a));
       description_b = g_content_type_get_description (content_type_b);
     }
 
@@ -1636,7 +1652,7 @@ thunar_list_model_set_case_sensitive (ThunarListModel *store,
       thunar_list_model_sort (store);
 
       /* notify listeners */
-      g_object_notify (G_OBJECT (store), "case-sensitive");
+      g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_CASE_SENSITIVE]);
     }
 }
 
@@ -1680,7 +1696,7 @@ thunar_list_model_set_date_style (ThunarListModel *store,
       store->date_style = date_style;
 
       /* notify listeners */
-      g_object_notify (G_OBJECT (store), "date-style");
+      g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_DATE_STYLE]);
 
       /* emit a "changed" signal for each row, so the display is reloaded with the new date style */
       gtk_tree_model_foreach (GTK_TREE_MODEL (store), (GtkTreeModelForeachFunc) gtk_tree_model_row_changed, NULL);
@@ -1797,8 +1813,8 @@ thunar_list_model_set_folder (ThunarListModel *store,
     }
 
   /* notify listeners that we have a new folder */
-  g_object_notify (G_OBJECT (store), "folder");
-  g_object_notify (G_OBJECT (store), "num-files");
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FOLDER]);
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_NUM_FILES]);
   g_object_thaw_notify (G_OBJECT (store));
 }
 
@@ -1839,7 +1855,7 @@ thunar_list_model_set_folders_first (ThunarListModel *store,
 
   /* apply the new setting (re-sorting the store) */
   store->sort_folders_first = folders_first;
-  g_object_notify (G_OBJECT (store), "folders-first");
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FOLDERS_FIRST]);
   thunar_list_model_sort (store);
 }
 
@@ -1941,8 +1957,8 @@ thunar_list_model_set_show_hidden (ThunarListModel *store,
 
   /* notify listeners about the new setting */
   g_object_freeze_notify (G_OBJECT (store));
-  g_object_notify (G_OBJECT (store), "num-files");
-  g_object_notify (G_OBJECT (store), "show-hidden");
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_NUM_FILES]);
+  g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_SHOW_HIDDEN]);
   g_object_thaw_notify (G_OBJECT (store));
 }
 
@@ -2128,7 +2144,6 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
   gint           folder_count;
   gint           non_folder_count;
   GList         *lp;
-  gchar         *absolute_path;
   gchar         *fspace_string;
   gchar         *display_name;
   gchar         *size_string;
@@ -2136,8 +2151,6 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
   gchar         *folder_text;
   gchar         *non_folder_text;
   gchar         *s;
-  gint           height;
-  gint           width;
   gchar         *description;
   GSequenceIter *row;
   GSequenceIter *end;
@@ -2252,22 +2265,6 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
           g_free (text);
           text = s;
         }
-      else if (thunar_file_is_local (file)
-               && thunar_file_is_regular (file)
-               && g_str_has_prefix (content_type, "image/")) /* bug #2913 */
-        {
-          /* check if we can determine the dimension of this file (only for image files) */
-          absolute_path = g_file_get_path (thunar_file_get_file (file));
-          if (absolute_path != NULL
-              && gdk_pixbuf_get_file_info (absolute_path, &width, &height) != NULL)
-            {
-              /* append the image dimensions to the statusbar text */
-              s = g_strdup_printf ("%s, %s %dx%d", text, _("Image Size:"), width, height);
-              g_free (text);
-              text = s;
-            }
-          g_free (absolute_path);
-        }
     }
   else
     {
